@@ -1,4 +1,5 @@
 use crate::{
+    custom::OptionsConfig,
     execute::{CodeExecuter, ExecutionHandle, ExecutionState, ProcessStatus},
     markdown::{
         elements::{
@@ -12,12 +13,17 @@ use crate::{
         PresentationThemeMetadata, RenderOnDemand, RenderOnDemandState, RenderOperation, Slide, SlideChunk,
     },
     render::{
-        highlighting::{CodeHighlighter, LanguageHighlighter, StyledTokens},
+        highlighting::{CodeHighlighter, HighlightThemeSet, LanguageHighlighter, StyledTokens},
+        media::Image,
         properties::WindowSize,
     },
     resource::{LoadImageError, Resources},
     style::{Colors, TextStyle},
-    theme::{Alignment, AuthorPositioning, ElementType, FooterStyle, LoadThemeError, Margin, PresentationTheme},
+    theme::{
+        Alignment, AuthorPositioning, ElementType, FooterStyle, LoadThemeError, Margin, PresentationTheme,
+        PresentationThemeSet,
+    },
+    typst::{TypstRender, TypstRenderError},
 };
 use itertools::Itertools;
 use serde::Deserialize;
@@ -28,13 +34,31 @@ use unicode_width::UnicodeWidthStr;
 // TODO: move to a theme config.
 static DEFAULT_BOTTOM_SLIDE_MARGIN: u16 = 3;
 
-pub(crate) struct PresentationBuilderOptions {
-    pub(crate) allow_mutations: bool,
+#[derive(Default)]
+pub struct Themes {
+    pub presentation: PresentationThemeSet,
+    pub highlight: HighlightThemeSet,
+}
+
+#[derive(Clone, Debug)]
+pub struct PresentationBuilderOptions {
+    pub allow_mutations: bool,
+    pub implicit_slide_ends: bool,
+    pub command_prefix: String,
+}
+
+impl PresentationBuilderOptions {
+    fn merge(&mut self, options: OptionsConfig) {
+        self.implicit_slide_ends = options.implicit_slide_ends.unwrap_or(self.implicit_slide_ends);
+        if let Some(prefix) = options.command_prefix {
+            self.command_prefix = prefix;
+        }
+    }
 }
 
 impl Default for PresentationBuilderOptions {
     fn default() -> Self {
-        Self { allow_mutations: true }
+        Self { allow_mutations: true, implicit_slide_ends: false, command_prefix: String::default() }
     }
 }
 
@@ -50,8 +74,10 @@ pub(crate) struct PresentationBuilder<'a> {
     highlighter: CodeHighlighter,
     theme: Cow<'a, PresentationTheme>,
     resources: &'a mut Resources,
+    typst: &'a mut TypstRender,
     slide_state: SlideState,
     footer_context: Rc<RefCell<FooterContext>>,
+    themes: &'a Themes,
     options: PresentationBuilderOptions,
 }
 
@@ -61,6 +87,8 @@ impl<'a> PresentationBuilder<'a> {
         default_highlighter: CodeHighlighter,
         default_theme: &'a PresentationTheme,
         resources: &'a mut Resources,
+        typst: &'a mut TypstRender,
+        themes: &'a Themes,
         options: PresentationBuilderOptions,
     ) -> Self {
         Self {
@@ -71,17 +99,26 @@ impl<'a> PresentationBuilder<'a> {
             highlighter: default_highlighter,
             theme: Cow::Borrowed(default_theme),
             resources,
+            typst,
             slide_state: Default::default(),
             footer_context: Default::default(),
+            themes,
             options,
         }
     }
 
     /// Build a presentation.
     pub(crate) fn build(mut self, elements: Vec<MarkdownElement>) -> Result<Presentation, BuildError> {
+        let mut skip_first = false;
         if let Some(MarkdownElement::FrontMatter(contents)) = elements.first() {
             self.process_front_matter(contents)?;
+            skip_first = true;
         }
+        let mut elements = elements.into_iter();
+        if skip_first {
+            elements.next();
+        }
+
         self.set_code_theme()?;
 
         if self.chunk_operations.is_empty() {
@@ -142,23 +179,26 @@ impl<'a> PresentationBuilder<'a> {
             MarkdownElement::Heading { level, text } => self.push_heading(level, text),
             MarkdownElement::Paragraph(elements) => self.push_paragraph(elements)?,
             MarkdownElement::List(elements) => self.push_list(elements),
-            MarkdownElement::Code(code) => self.push_code(code),
+            MarkdownElement::Code(code) => self.push_code(code)?,
             MarkdownElement::Table(table) => self.push_table(table),
             MarkdownElement::ThematicBreak => self.push_separator(),
             MarkdownElement::Comment { comment, source_position } => self.process_comment(comment, source_position)?,
             MarkdownElement::BlockQuote(lines) => self.push_block_quote(lines),
-            MarkdownElement::Image { path, .. } => self.push_image(path)?,
+            MarkdownElement::Image { path, .. } => self.push_image_from_path(path)?,
         };
         if should_clear_last {
-            self.slide_state.last_element = Default::default();
+            self.slide_state.last_element = LastElement::Other;
         }
         Ok(())
     }
 
     fn process_front_matter(&mut self, contents: &str) -> Result<(), BuildError> {
-        let metadata: PresentationMetadata =
+        let mut metadata: PresentationMetadata =
             serde_yaml::from_str(contents).map_err(|e| BuildError::InvalidMetadata(e.to_string()))?;
 
+        if let Some(options) = metadata.options.take() {
+            self.options.merge(options);
+        }
         self.footer_context.borrow_mut().author = metadata.author.clone().unwrap_or_default();
         self.set_theme(&metadata.theme)?;
         if metadata.title.is_some() || metadata.sub_title.is_some() || metadata.author.is_some() {
@@ -173,7 +213,10 @@ impl<'a> PresentationBuilder<'a> {
             return Err(BuildError::InvalidMetadata("cannot have both theme path and theme name".into()));
         }
         if let Some(theme_name) = &metadata.name {
-            let theme = PresentationTheme::from_name(theme_name)
+            let theme = self
+                .themes
+                .presentation
+                .load_by_name(theme_name)
                 .ok_or_else(|| BuildError::InvalidMetadata(format!("theme '{theme_name}' does not exist")))?;
             self.theme = Cow::Owned(theme);
         }
@@ -192,7 +235,8 @@ impl<'a> PresentationBuilder<'a> {
 
     fn set_code_theme(&mut self) -> Result<(), BuildError> {
         if let Some(theme) = &self.theme.code.theme_name {
-            let highlighter = CodeHighlighter::new(theme).map_err(|_| BuildError::InvalidCodeTheme)?;
+            let highlighter =
+                self.themes.highlight.load_by_name(theme).ok_or_else(|| BuildError::InvalidCodeTheme(theme.clone()))?;
             self.highlighter = highlighter;
         }
         Ok(())
@@ -236,9 +280,11 @@ impl<'a> PresentationBuilder<'a> {
     }
 
     fn process_comment(&mut self, comment: String, source_position: SourcePosition) -> Result<(), BuildError> {
-        if Self::should_ignore_comment(&comment) {
+        let comment = comment.trim();
+        if self.should_ignore_comment(comment) {
             return Ok(());
         }
+        let comment = comment.trim_start_matches(&self.options.command_prefix);
         let comment = match comment.parse::<CommentCommand>() {
             Ok(comment) => comment,
             Err(error) => return Err(BuildError::CommandParse { line: source_position.start.line + 1, error }),
@@ -246,6 +292,7 @@ impl<'a> PresentationBuilder<'a> {
         match comment {
             CommentCommand::Pause => self.process_pause(),
             CommentCommand::EndSlide => self.terminate_slide(),
+            CommentCommand::JumpToMiddle => self.chunk_operations.push(RenderOperation::JumpToVerticalCenter),
             CommentCommand::InitColumnLayout(columns) => {
                 Self::validate_column_layout(&columns)?;
                 self.slide_state.layout = LayoutState::InLayout { columns_count: columns.len() };
@@ -276,14 +323,16 @@ impl<'a> PresentationBuilder<'a> {
         Ok(())
     }
 
-    fn should_ignore_comment(comment: &str) -> bool {
-        // Ignore any multi line comment; those are assumed to be user comments
-        if comment.contains('\n') {
-            return true;
+    fn should_ignore_comment(&self, comment: &str) -> bool {
+        if comment.contains('\n') || !comment.starts_with(&self.options.command_prefix) {
+            // Ignore any multi line comment; those are assumed to be user comments
+            // Ignore any line that doesn't start with the selected prefix.
+            true
+        } else {
+            // Ignore vim-like code folding tags
+            let comment = comment.trim();
+            comment == "{{{" || comment == "}}}"
         }
-        // Ignore vim-like code folding tags
-        let comment = comment.trim();
-        comment == "{{{" || comment == "}}}"
     }
 
     fn validate_column_layout(columns: &[u8]) -> Result<(), BuildError> {
@@ -305,6 +354,10 @@ impl<'a> PresentationBuilder<'a> {
     }
 
     fn push_slide_title(&mut self, mut text: Text) {
+        if self.options.implicit_slide_ends && !matches!(self.slide_state.last_element, LastElement::None) {
+            self.terminate_slide();
+        }
+
         let style = self.theme.slide_title.clone();
         text.apply_style(&TextStyle::default().bold().colors(style.colors.clone()));
 
@@ -365,11 +418,15 @@ impl<'a> PresentationBuilder<'a> {
         self.chunk_operations.extend([RenderSeparator::default().into(), RenderOperation::RenderLineBreak]);
     }
 
-    fn push_image(&mut self, path: PathBuf) -> Result<(), BuildError> {
+    fn push_image_from_path(&mut self, path: PathBuf) -> Result<(), BuildError> {
         let image = self.resources.image(&path)?;
+        self.push_image(image);
+        Ok(())
+    }
+
+    fn push_image(&mut self, image: Image) {
         self.chunk_operations.push(RenderOperation::RenderImage(image));
         self.chunk_operations.push(RenderOperation::SetColors(self.theme.default_style.colors.clone()));
-        Ok(())
     }
 
     fn push_list(&mut self, list: Vec<ListItem>) {
@@ -470,17 +527,32 @@ impl<'a> PresentationBuilder<'a> {
         self.chunk_operations.push(RenderOperation::RenderLineBreak);
     }
 
-    fn push_code(&mut self, code: Code) {
+    fn push_code(&mut self, code: Code) -> Result<(), BuildError> {
+        if code.attributes.auto_render {
+            return self.push_rendered_code(code);
+        }
         let (lines, context) = self.highlight_lines(&code);
         for line in lines {
             self.chunk_operations.push(RenderOperation::RenderDynamic(Rc::new(line)));
         }
+        self.chunk_operations.push(RenderOperation::SetColors(self.theme.default_style.colors.clone()));
         if self.options.allow_mutations && context.borrow().groups.len() > 1 {
             self.chunk_mutators.push(Box::new(HighlightMutator { context }));
         }
         if code.attributes.execute {
             self.push_code_execution(code);
         }
+        Ok(())
+    }
+
+    fn push_rendered_code(&mut self, code: Code) -> Result<(), BuildError> {
+        let image = match code.language {
+            CodeLanguage::Typst => self.typst.render_typst(&code.contents, &self.theme.typst)?,
+            CodeLanguage::Latex => self.typst.render_latex(&code.contents, &self.theme.typst)?,
+            _ => panic!("language {:?} should not be renderable", code.language),
+        };
+        self.push_image(image);
+        Ok(())
     }
 
     fn highlight_lines(&self, code: &Code) -> (Vec<HighlightedLine>, Rc<RefCell<HighlightContext>>) {
@@ -490,7 +562,7 @@ impl<'a> PresentationBuilder<'a> {
         let mut code_highlighter = self.highlighter.language_highlighter(&code.language);
         let padding_style = {
             let mut highlighter = self.highlighter.language_highlighter(&CodeLanguage::Rust);
-            highlighter.style_line("//").first().expect("no styles").style
+            highlighter.style_line("//").next().expect("no styles").style
         };
         let groups = match self.options.allow_mutations {
             true => code.attributes.highlight_groups.clone(),
@@ -536,6 +608,7 @@ impl<'a> PresentationBuilder<'a> {
         self.slides.push(Slide::new(chunks, footer));
         self.push_slide_prelude();
         self.slide_state = Default::default();
+        self.slide_state.last_element = LastElement::None;
     }
 
     fn generate_footer(&mut self) -> Vec<RenderOperation> {
@@ -669,8 +742,6 @@ impl CodeLine {
     fn highlight(&self, padding_style: &Style, code_highlighter: &mut LanguageHighlighter) -> String {
         let mut output = StyledTokens { style: *padding_style, tokens: &self.prefix }.apply_style();
         output.push_str(&code_highlighter.highlight_line(&self.code));
-        // Remove newline
-        output.pop();
         output.push_str(&StyledTokens { style: *padding_style, tokens: &self.suffix }.apply_style());
         output
     }
@@ -785,10 +856,11 @@ enum LayoutState {
 #[derive(Debug, Default)]
 enum LastElement {
     #[default]
-    Any,
+    None,
     List {
         last_index: usize,
     },
+    Other,
 }
 
 #[derive(Debug, Default)]
@@ -884,8 +956,8 @@ pub enum BuildError {
     #[error("invalid theme: {0}")]
     InvalidTheme(#[from] LoadThemeError),
 
-    #[error("invalid code highlighter theme")]
-    InvalidCodeTheme,
+    #[error("invalid code highlighter theme: '{0}'")]
+    InvalidCodeTheme(String),
 
     #[error("invalid layout: {0}")]
     InvalidLayout(&'static str),
@@ -904,6 +976,9 @@ pub enum BuildError {
 
     #[error("error parsing command at line {line}: {error}")]
     CommandParse { line: usize, error: CommandParseError },
+
+    #[error("typst render failed: {0}")]
+    TypstRender(#[from] TypstRenderError),
 }
 
 #[derive(Debug, Clone, PartialEq, Deserialize)]
@@ -915,6 +990,7 @@ enum CommentCommand {
     InitColumnLayout(Vec<u8>),
     Column(usize),
     ResetLayout,
+    JumpToMiddle,
 }
 
 impl FromStr for CommentCommand {
@@ -1146,12 +1222,27 @@ mod test {
         try_build_presentation(elements).expect("build failed")
     }
 
+    fn build_presentation_with_options(
+        elements: Vec<MarkdownElement>,
+        options: PresentationBuilderOptions,
+    ) -> Presentation {
+        try_build_presentation_with_options(elements, options).expect("build failed")
+    }
+
     fn try_build_presentation(elements: Vec<MarkdownElement>) -> Result<Presentation, BuildError> {
-        let highlighter = CodeHighlighter::new("base16-ocean.dark").unwrap();
+        try_build_presentation_with_options(elements, Default::default())
+    }
+
+    fn try_build_presentation_with_options(
+        elements: Vec<MarkdownElement>,
+        options: PresentationBuilderOptions,
+    ) -> Result<Presentation, BuildError> {
+        let highlighter = CodeHighlighter::default();
         let theme = PresentationTheme::default();
         let mut resources = Resources::new("/tmp");
-        let options = PresentationBuilderOptions::default();
-        let builder = PresentationBuilder::new(highlighter, &theme, &mut resources, options);
+        let mut typst = TypstRender::default();
+        let themes = Themes::default();
+        let builder = PresentationBuilder::new(highlighter, &theme, &mut resources, &mut typst, &themes, options);
         builder.build(elements)
     }
 
@@ -1431,12 +1522,55 @@ mod test {
         assert!(matches!(last_operation, RenderOperation::RenderLineBreak), "last operation is {last_operation:?}");
     }
 
+    #[test]
+    fn implicit_slide_ends() {
+        let elements = vec![
+            // first slide
+            MarkdownElement::SetexHeading { text: "hi".into() },
+            // second
+            MarkdownElement::SetexHeading { text: "hi".into() },
+            MarkdownElement::Heading { level: 1, text: "hi".into() },
+            // explicitly ends
+            MarkdownElement::Comment { comment: "end_slide".into(), source_position: Default::default() },
+            // third starts
+            MarkdownElement::SetexHeading { text: "hi".into() },
+        ];
+        let options = PresentationBuilderOptions { implicit_slide_ends: true, ..Default::default() };
+        let slides = build_presentation_with_options(elements, options).into_slides();
+        assert_eq!(slides.len(), 3);
+    }
+
+    #[test]
+    fn implicit_slide_ends_with_front_matter() {
+        let elements = vec![
+            MarkdownElement::FrontMatter("theme:\n name: light".into()),
+            MarkdownElement::SetexHeading { text: "hi".into() },
+        ];
+        let options = PresentationBuilderOptions { implicit_slide_ends: true, ..Default::default() };
+        let slides = build_presentation_with_options(elements, options).into_slides();
+        assert_eq!(slides.len(), 1);
+    }
+
     #[rstest]
     #[case::multiline("hello\nworld")]
     #[case::many_open_braces("{{{")]
     #[case::many_close_braces("}}}")]
     fn ignore_comments(#[case] comment: &str) {
-        assert!(PresentationBuilder::should_ignore_comment(comment));
+        let element = MarkdownElement::Comment { comment: comment.into(), source_position: Default::default() };
+        build_presentation(vec![element]);
+    }
+
+    #[rstest]
+    #[case::command_with_prefix("cmd:end_slide", true)]
+    #[case::non_command_with_prefix("cmd:bogus", false)]
+    #[case::non_prefixed("random", true)]
+    fn comment_prefix(#[case] comment: &str, #[case] should_work: bool) {
+        let mut options = PresentationBuilderOptions::default();
+        options.command_prefix = "cmd:".into();
+
+        let element = MarkdownElement::Comment { comment: comment.into(), source_position: Default::default() };
+        let result = try_build_presentation_with_options(vec![element], options);
+        assert_eq!(result.is_ok(), should_work, "{result:?}");
     }
 
     #[test]
