@@ -3,6 +3,7 @@
 //! This is not meant to be used as a crate!
 
 pub(crate) mod builder;
+pub(crate) mod custom;
 pub(crate) mod diff;
 pub(crate) mod execute;
 pub(crate) mod export;
@@ -14,13 +15,18 @@ pub(crate) mod render;
 pub(crate) mod resource;
 pub(crate) mod style;
 pub(crate) mod theme;
+pub(crate) mod tools;
+pub(crate) mod typst;
 
 pub use crate::{
+    builder::{PresentationBuilderOptions, Themes},
+    custom::Config,
     export::{ExportError, Exporter},
     input::source::CommandSource,
     markdown::parse::MarkdownParser,
     presenter::{PresentMode, Presenter},
-    render::highlighting::CodeHighlighter,
+    render::highlighting::{CodeHighlighter, HighlightThemeSet},
     resource::Resources,
-    theme::PresentationTheme,
+    theme::{LoadThemeError, PresentationTheme, PresentationThemeSet},
+    typst::TypstRender,
 };
