@@ -1,9 +1,13 @@
 use clap::{error::ErrorKind, CommandFactory, Parser};
 use comrak::Arena;
 use presenterm::{
-    CodeHighlighter, CommandSource, Exporter, MarkdownParser, PresentMode, PresentationTheme, Presenter, Resources,
+    CommandSource, Config, Exporter, HighlightThemeSet, LoadThemeError, MarkdownParser, PresentMode,
+    PresentationBuilderOptions, PresentationThemeSet, Presenter, Resources, Themes, TypstRender,
+};
+use std::{
+    env,
+    path::{Path, PathBuf},
 };
-use std::path::{Path, PathBuf};
 
 /// Run slideshows from your terminal.
 #[derive(Parser)]
@@ -46,23 +50,61 @@ fn create_splash() -> String {
         r#"
   ┌─┐┬─┐┌─┐┌─┐┌─┐┌┐┌┌┬┐┌─┐┬─┐┌┬┐
   ├─┘├┬┘├┤ └─┐├┤ │││ │ ├┤ ├┬┘│││
-  ┴  ┴└─└─┘└─┘└─┘┘└┘ ┴ └─┘┴└─┴ ┴ v{}
+  ┴  ┴└─└─┘└─┘└─┘┘└┘ ┴ └─┘┴└─┴ ┴ v{crate_version}
     A terminal slideshow tool 
                     @mfontanini/presenterm
 "#,
-        crate_version,
     )
 }
 
+fn load_customizations() -> Result<(Config, Themes), Box<dyn std::error::Error>> {
+    let Ok(home_path) = env::var("HOME") else {
+        return Ok(Default::default());
+    };
+    let home_path = PathBuf::from(home_path);
+    let configs_path = home_path.join(".config/presenterm");
+    let themes = load_themes(&configs_path)?;
+    let config_file_path = configs_path.join("config.yaml");
+    let config = Config::load(&config_file_path)?;
+    Ok((config, themes))
+}
+
+fn load_themes(config_path: &Path) -> Result<Themes, Box<dyn std::error::Error>> {
+    let themes_path = config_path.join("themes");
+
+    let mut highlight_themes = HighlightThemeSet::default();
+    highlight_themes.register_from_directory(&themes_path.join("highlighting"))?;
+
+    let mut presentation_themes = PresentationThemeSet::default();
+    let register_result = presentation_themes.register_from_directory(&themes_path);
+    if let Err(e @ (LoadThemeError::Duplicate(_) | LoadThemeError::Corrupted(..))) = register_result {
+        return Err(e.into());
+    }
+
+    let themes = Themes { presentation: presentation_themes, highlight: highlight_themes };
+    Ok(themes)
+}
+
 fn display_acknowledgements() {
     let acknowledgements = include_bytes!("../bat/acknowledgements.txt");
     println!("{}", String::from_utf8_lossy(acknowledgements));
 }
 
+fn make_builder_options(config: &Config, mode: &PresentMode) -> PresentationBuilderOptions {
+    PresentationBuilderOptions {
+        allow_mutations: !matches!(mode, PresentMode::Export),
+        implicit_slide_ends: config.options.implicit_slide_ends.unwrap_or_default(),
+        command_prefix: config.options.command_prefix.clone().unwrap_or_default(),
+    }
+}
+
 fn run(cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
-    let Some(default_theme) = PresentationTheme::from_name(&cli.theme) else {
+    let (config, themes) = load_customizations()?;
+
+    let default_theme_name = config.defaults.theme.as_ref().unwrap_or(&cli.theme);
+    let Some(default_theme) = themes.presentation.load_by_name(default_theme_name) else {
         let mut cmd = Cli::command();
-        let valid_themes = PresentationTheme::theme_names().collect::<Vec<_>>().join(", ");
+        let valid_themes = themes.presentation.theme_names().join(", ");
         let error_message = format!("invalid theme name, valid themes are: {valid_themes}");
         cmd.error(ErrorKind::InvalidValue, error_message).exit();
     };
@@ -74,7 +116,6 @@ fn run(cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
     };
     let arena = Arena::new();
     let parser = MarkdownParser::new(&arena);
-    let default_highlighter = CodeHighlighter::new("base16-ocean.dark")?;
     if cli.acknowledgements {
         display_acknowledgements();
         return Ok(());
@@ -82,8 +123,10 @@ fn run(cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
     let path = cli.path.expect("no path");
     let resources_path = path.parent().unwrap_or(Path::new("/"));
     let resources = Resources::new(resources_path);
+    let typst = TypstRender::new(config.typst.ppi);
+    let options = make_builder_options(&config, &mode);
     if cli.export_pdf || cli.generate_pdf_metadata {
-        let mut exporter = Exporter::new(parser, &default_theme, default_highlighter, resources);
+        let mut exporter = Exporter::new(parser, &default_theme, resources, typst, themes, options);
         if cli.export_pdf {
             exporter.export_pdf(&path)?;
         } else {
@@ -92,7 +135,7 @@ fn run(cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
         }
     } else {
         let commands = CommandSource::new(&path);
-        let presenter = Presenter::new(&default_theme, default_highlighter, commands, parser, resources, mode);
+        let presenter = Presenter::new(&default_theme, commands, parser, resources, typst, themes, mode, options);
         presenter.present(&path)?;
     }
     Ok(())
