@@ -1,4 +1,5 @@
 use crate::{
+    custom::OptionsConfig,
     markdown::text::WeightedLine,
     render::{media::Image, properties::WindowSize},
     style::Colors,
@@ -8,6 +9,7 @@ use serde::Deserialize;
 use std::{fmt::Debug, rc::Rc};
 
 /// A presentation.
+#[derive(Debug)]
 pub(crate) struct Presentation {
     slides: Vec<Slide>,
     current_slide_index: usize,
@@ -129,6 +131,22 @@ impl Presentation {
         all_rendered
     }
 
+    /// Run a callback through every operation and let it mutate it in place.
+    ///
+    /// This should be used with care!
+    pub(crate) fn mutate_operations<F>(&mut self, mut callback: F)
+    where
+        F: FnMut(&mut RenderOperation),
+    {
+        for slide in &mut self.slides {
+            for chunk in &mut slide.chunks {
+                for operation in &mut chunk.operations {
+                    callback(operation);
+                }
+            }
+        }
+    }
+
     fn current_slide_mut(&mut self) -> &mut Slide {
         &mut self.slides[self.current_slide_index]
     }
@@ -307,6 +325,10 @@ pub(crate) struct PresentationMetadata {
     /// The presentation's theme metadata.
     #[serde(default)]
     pub(crate) theme: PresentationThemeMetadata,
+
+    /// The presentation's options.
+    #[serde(default)]
+    pub(crate) options: Option<OptionsConfig>,
 }
 
 /// A presentation's theme metadata.
