@@ -1,6 +1,7 @@
 use crate::{
-    builder::{BuildError, PresentationBuilder, PresentationBuilderOptions},
+    builder::{BuildError, PresentationBuilder, PresentationBuilderOptions, Themes},
     diff::PresentationDiffer,
+    export::ImageReplacer,
     input::source::{Command, CommandSource},
     markdown::parse::{MarkdownParser, ParseError},
     presentation::Presentation,
@@ -10,6 +11,7 @@ use crate::{
     },
     resource::Resources,
     theme::PresentationTheme,
+    typst::TypstRender,
 };
 use std::{
     collections::HashSet,
@@ -24,34 +26,41 @@ use std::{
 /// This type puts everything else together.
 pub struct Presenter<'a> {
     default_theme: &'a PresentationTheme,
-    default_highlighter: CodeHighlighter,
     commands: CommandSource,
     parser: MarkdownParser<'a>,
     resources: Resources,
+    typst: TypstRender,
     mode: PresentMode,
     state: PresenterState,
     slides_with_pending_widgets: HashSet<usize>,
+    themes: Themes,
+    builder_options: PresentationBuilderOptions,
 }
 
 impl<'a> Presenter<'a> {
     /// Construct a new presenter.
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         default_theme: &'a PresentationTheme,
-        default_highlighter: CodeHighlighter,
         commands: CommandSource,
         parser: MarkdownParser<'a>,
         resources: Resources,
+        typst: TypstRender,
+        themes: Themes,
         mode: PresentMode,
+        builder_options: PresentationBuilderOptions,
     ) -> Self {
         Self {
             default_theme,
-            default_highlighter,
             commands,
             parser,
             resources,
+            typst,
             mode,
             state: PresenterState::Empty,
             slides_with_pending_widgets: HashSet::new(),
+            themes,
+            builder_options,
         }
     }
 
@@ -179,17 +188,20 @@ impl<'a> Presenter<'a> {
     fn load_presentation(&mut self, path: &Path) -> Result<Presentation, LoadPresentationError> {
         let content = fs::read_to_string(path).map_err(LoadPresentationError::Reading)?;
         let elements = self.parser.parse(&content)?;
-        let mut options = PresentationBuilderOptions::default();
-        if matches!(self.mode, PresentMode::Export) {
-            options.allow_mutations = false;
-        }
-        let presentation = PresentationBuilder::new(
-            self.default_highlighter.clone(),
+        let export_mode = matches!(self.mode, PresentMode::Export);
+        let mut presentation = PresentationBuilder::new(
+            CodeHighlighter::default(),
             self.default_theme,
             &mut self.resources,
-            options,
+            &mut self.typst,
+            &self.themes,
+            self.builder_options.clone(),
         )
         .build(elements)?;
+        if export_mode {
+            ImageReplacer::default().replace_presentation_images(&mut presentation);
+        }
+
         Ok(presentation)
     }
 }
