@@ -1,6 +1,6 @@
 use crate::render::properties::WindowSize;
 use image::{DynamicImage, ImageError};
-use std::{fmt::Debug, io, path::PathBuf, rc::Rc};
+use std::{fmt::Debug, io, ops::Deref, path::PathBuf, rc::Rc};
 use viuer::{is_iterm_supported, ViuError};
 
 use super::properties::CursorPosition;
@@ -11,7 +11,7 @@ use super::properties::CursorPosition;
 #[derive(Clone, PartialEq)]
 pub(crate) struct Image {
     contents: Rc<DynamicImage>,
-    path: PathBuf,
+    pub(crate) source: ImageSource,
 }
 
 impl Debug for Image {
@@ -21,14 +21,33 @@ impl Debug for Image {
 }
 
 impl Image {
-    /// Construct a new image from a byte sequence.
-    pub(crate) fn new(contents: &[u8], path: PathBuf) -> Result<Self, InvalidImage> {
+    /// Constructs a new image.
+    pub(crate) fn new(image: DynamicImage) -> Self {
+        Self { contents: Rc::new(image), source: ImageSource::Generated }
+    }
+
+    /// Decodes an image from a byte sequence.
+    pub(crate) fn decode(contents: &[u8], source: ImageSource) -> Result<Self, InvalidImage> {
         let contents = image::load_from_memory(contents)?;
         let contents = Rc::new(contents);
-        Ok(Self { contents, path })
+        Ok(Self { contents, source })
     }
 }
 
+impl Deref for Image {
+    type Target = DynamicImage;
+
+    fn deref(&self) -> &Self::Target {
+        &self.contents
+    }
+}
+
+#[derive(Clone, Debug, PartialEq)]
+pub(crate) enum ImageSource {
+    Filesystem(PathBuf),
+    Generated,
+}
+
 /// A media render.
 pub(crate) struct MediaRender {
     mode: TerminalMode,
@@ -52,7 +71,7 @@ impl MediaRender {
         if !dimensions.has_pixels {
             return Err(RenderImageError::NoWindowSize);
         }
-        let image_path = &image.path;
+        let source = &image.source;
         let image = &image.contents;
 
         // Compute the image's width in columns by translating pixels -> columns.
@@ -70,7 +89,7 @@ impl MediaRender {
             // Because we only use the width to draw, here we scale the width based on how much we
             // need to shrink the height.
             let shrink_ratio = available_height as f64 / height_in_rows as f64;
-            width_in_columns = (width_in_columns as f64 * shrink_ratio) as u32;
+            width_in_columns = (width_in_columns as f64 * shrink_ratio).ceil() as u32;
         }
         // Don't go too far wide.
         let width_in_columns = width_in_columns.min(column_margin);
@@ -89,9 +108,9 @@ impl MediaRender {
         //
         // This switch is because otherwise `viuer::print_from_file` for kitty/ascii blocks will
         // re-read the image every time.
-        match self.mode {
-            TerminalMode::Iterm2 => viuer::print_from_file(image_path, &config)?,
-            TerminalMode::Other => viuer::print(image, &config)?,
+        match (&self.mode, source) {
+            (TerminalMode::Iterm2, ImageSource::Filesystem(image_path)) => viuer::print_from_file(image_path, &config)?,
+            (TerminalMode::Other, _) | (_, ImageSource::Generated) => viuer::print(image, &config)?,
         };
         Ok(())
     }
