@@ -4,6 +4,55 @@ use std::{fs, io, path::Path};
 
 include!(concat!(env!("OUT_DIR"), "/themes.rs"));
 
+#[derive(Default)]
+pub struct PresentationThemeSet {
+    custom_themes: HashMap<String, PresentationTheme>,
+}
+
+impl PresentationThemeSet {
+    /// Loads a theme from its name.
+    pub fn load_by_name(&self, name: &str) -> Option<PresentationTheme> {
+        if let Some(contents) = THEMES.get(name) {
+            // This is going to be caught by the test down here.
+            Some(serde_yaml::from_slice(contents).expect("corrupted theme"))
+        } else {
+            self.custom_themes.get(name).cloned()
+        }
+    }
+
+    /// Register all the themes in the given directory.
+    pub fn register_from_directory<P: AsRef<Path>>(&mut self, path: P) -> Result<(), LoadThemeError> {
+        let handle = match fs::read_dir(&path) {
+            Ok(handle) => handle,
+            Err(e) if e.kind() == io::ErrorKind::NotFound => return Ok(()),
+            Err(e) => return Err(e.into()),
+        };
+        for entry in handle {
+            let entry = entry?;
+            let metadata = entry.metadata()?;
+            let Some(file_name) = entry.file_name().to_str().map(ToOwned::to_owned) else {
+                continue;
+            };
+            if metadata.is_file() && file_name.ends_with(".yaml") {
+                let theme_name = file_name.trim_end_matches(".yaml");
+                if THEMES.contains_key(theme_name) {
+                    return Err(LoadThemeError::Duplicate(theme_name.into()));
+                }
+                let theme = PresentationTheme::from_path(&entry.path())?;
+                self.custom_themes.insert(theme_name.into(), theme);
+            }
+        }
+        Ok(())
+    }
+
+    /// Get all the registered theme names.
+    pub fn theme_names(&self) -> Vec<String> {
+        let builtin_themes = THEMES.keys().map(|name| name.to_string());
+        let themes = self.custom_themes.keys().cloned().chain(builtin_themes).collect();
+        themes
+    }
+}
+
 /// A presentation theme.
 #[derive(Default, Clone, Debug, Deserialize, Serialize)]
 pub struct PresentationTheme {
@@ -46,27 +95,18 @@ pub struct PresentationTheme {
     /// The style of the presentation footer.
     #[serde(default)]
     pub(crate) footer: Option<FooterStyle>,
+
+    /// The style for typst auto-rendered code blocks.
+    #[serde(default)]
+    pub(crate) typst: TypstStyle,
 }
 
 impl PresentationTheme {
-    /// Get a presentation theme by name.
-    ///
-    /// Default themes are bundled into the final binary during build time so this is an in-memory
-    /// lookup.
-    pub fn from_name(name: &str) -> Option<Self> {
-        let contents = THEMES.get(name)?;
-        // This is going to be caught by the test down here.
-        Some(serde_yaml::from_slice(contents).expect("corrupted theme"))
-    }
-
-    pub fn theme_names() -> impl Iterator<Item = &'static str> {
-        THEMES.keys().copied()
-    }
-
     /// Construct a presentation from a path.
     pub(crate) fn from_path<P: AsRef<Path>>(path: P) -> Result<Self, LoadThemeError> {
-        let contents = fs::read_to_string(path)?;
-        let theme = serde_yaml::from_str(&contents)?;
+        let contents = fs::read_to_string(&path)?;
+        let theme = serde_yaml::from_str(&contents)
+            .map_err(|e| LoadThemeError::Corrupted(path.as_ref().display().to_string(), e))?;
         Ok(theme)
     }
 
@@ -429,24 +469,42 @@ pub(crate) enum AuthorPositioning {
     PageBottom,
 }
 
+/// Where to position the author's name in the intro slide.
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
+pub(crate) struct TypstStyle {
+    /// The horizontal margin on the generated images.
+    pub(crate) horizontal_margin: Option<u16>,
+
+    /// The vertical margin on the generated images.
+    pub(crate) vertical_margin: Option<u16>,
+
+    /// The colors to be used.
+    pub(crate) colors: Colors,
+}
+
 /// An error loading a presentation theme.
 #[derive(thiserror::Error, Debug)]
 pub enum LoadThemeError {
     #[error(transparent)]
     Io(#[from] io::Error),
 
-    #[error(transparent)]
-    Corrupted(#[from] serde_yaml::Error),
+    #[error("theme at '{0}' is corrupted: {1}")]
+    Corrupted(String, serde_yaml::Error),
+
+    #[error("duplicate custom theme '{0}'")]
+    Duplicate(String),
 }
 
 #[cfg(test)]
 mod test {
     use super::*;
+    use tempfile::tempdir;
 
     #[test]
     fn validate_themes() {
+        let themes = PresentationThemeSet::default();
         for theme_name in THEMES.keys() {
-            let Some(theme) = PresentationTheme::from_name(theme_name) else {
+            let Some(theme) = themes.load_by_name(theme_name) else {
                 panic!("theme '{theme_name}' is corrupted");
             };
 
@@ -454,4 +512,22 @@ mod test {
             assert!(merged.is_ok(), "theme '{theme_name}' can't be merged: {}", merged.unwrap_err());
         }
     }
+
+    #[test]
+    fn load_custom() {
+        let directory = tempdir().expect("creating tempdir");
+        let theme = serde_yaml::to_string(&PresentationTheme::default()).unwrap();
+        fs::write(directory.path().join("potato.yaml"), &theme).expect("writing theme");
+
+        let mut themes = PresentationThemeSet::default();
+        themes.register_from_directory(directory.path()).expect("loading themes");
+        assert!(themes.load_by_name("potato").is_some());
+    }
+
+    #[test]
+    fn register_from_missing_directory() {
+        let mut themes = PresentationThemeSet::default();
+        let result = themes.register_from_directory("/tmp/presenterm/8ee2027983915ec78acc45027d874316");
+        result.expect("loading failed");
+    }
 }
